<?PHP

/**************************************
    Webutler V3.7 - www.webutler.de
    Copyright (c) 2008 - 2025
    Autor: Sven Zinke
    Free for any use
    Lizenz: GPL
**************************************/


if(preg_match('#/includes/sitemap/index.php#i', $_SERVER['REQUEST_URI']))
    exit('no access');


require_once dirname(dirname(__FILE__)).'/loader.php';
require_once $webutler_config['server_path'].'/settings/sitemaps.php';
if(isset($webutler_modulesitemaps) && count($webutler_modulesitemaps) > 0)
	require_once $webutler_config['server_path'].'/includes/mmclass.php';

header("Content-type: text/xml");

class SitemapXML extends WebutlerClass {
    
    var $config;
    var $offlinepages;
    var $langconf;
    var $directories;
    var $blockedpages;
	var $modulesitemaps;
	var $modparenturls = array();
    
	function __construct()
	{
	}

    function blockedpages()
    {
        $blockedpages = array();
        
        if(file_exists($this->config['server_path'].'/content/access/users.db') && class_exists('SQLite3'))
        {
            $db = new SQLite3($this->config['server_path'].'/content/access/users.db');
            
            $blocks = $db->query("SELECT id, pages FROM blocks WHERE blocks.id = '1' LIMIT 1");
        	if($block = $blocks->fetchArray()) {
                $blockedpages = explode(',', $block['pages']);
            }
        }
		
		$pagesdir = $this->config['server_path'].'/content/pages';
		$handle = opendir($pagesdir);
		while(false !== ($page = readdir($handle)))
		{
			$ext = substr($page, -4);
			if(!is_dir($pagesdir.'/'.$page.'/') && $page != '.' && $page != '..' && $page != '.htaccess' && $ext != '.bak' && $ext != '.tmp' && $page != $this->config['errorpage'] && !in_array($page, $this->offlinepages))
			{
				$metatags = get_meta_tags($pagesdir.'/'.$page);
				
				if(isset($metatags['robots']) && preg_match('#noindex#i', $metatags['robots']))
					$blockedpages[] = $page;
			}
		}
		
        return $blockedpages;
	}
    
	function getpages()
	{
		$this->blockedpages = $this->blockedpages();
		
		$results = array();
		
		$modulepages = $this->modulepages();
		$contentpages = $this->contentpages();
		
		if(array_key_exists('code', $this->langconf)) {
			foreach($this->langconf['code'] as $code) {
				if(!isset($contentpages[$code])) $contentpages[$code] = array();
				if(!isset($modulepages[$code])) $modulepages[$code] = array();
				$results[$code] = array_merge($contentpages[$code], $modulepages[$code]);
				
				if(count($results[$code]) > 0) {
					$tmp = array();
					foreach ($results[$code] as &$result) {
						$tmp[] = &$result['priority'];
					}
					
					array_multisort($tmp, SORT_DESC, $results[$code]);
				}
			}
		}
		else {
			$results = array_merge($contentpages, $modulepages);
			
			if(count($results) > 0) {
				$tmp = array();
				foreach ($results as &$result) {
					$tmp[] = &$result['priority'];
				}
				
				array_multisort($tmp, SORT_DESC, $results);
			}
		}
		
		unset($contentpages);
		unset($modulepages);
		
		return $results;
	}
	
    function modulepages()
    {
		$modulepages = array();
		
		if(count($this->modulesitemaps) > 0) {
			$directories = '';
			if($this->config['directories'] == '1' && array_key_exists('pages', $this->directories) && count($this->directories['pages']) > 0) {
				$directories = $this->directories['pages'];
			}
			
			foreach($this->modulesitemaps as $modules) {
				$geturls = $this->getmodulessitemap($modules);
				
				$countparenturls = count($this->modparenturls);
				$priority_data = '0.8';
				if($countparenturls > 0) {
					$priority_parents = '0.7';
				}
				
				if(array_key_exists('code', $this->langconf)) {
					if($countparenturls > 0) {
						foreach($this->modparenturls as $lang => $modurls) {
							if($lang != 'root') {
								foreach($modurls as $modurl) {
									if(is_array($modurl) && array_key_exists('location', $modurl)) {
										$url = $modurl['location'];
										$modified = isset($modurl['modified']) ? $modurl['modified'] : '';
										$hreflangs = array();
										if(isset($modurl['hreflangs']) && count($modurl['hreflangs']) > 0) {
											foreach($modurl['hreflangs'] as $rellang => $rel)
											{
												if($this->config['modrewrite'] == '1') {
													$relurl = explode('&', $rel);
													$relpage = $relurl[0];
													unset($relurl);
													
													$checkrel = $this->setlangasdirectory($relpage);
													$relfolder = $checkrel[0] != '' ? '/'.$checkrel[0] : '';
													if($checkrel[1] == '') $checkrel[1] = 'index';
													$hrefurl = $relpage != $checkrel[1] ? $checkrel[1].substr($rel, strlen($relpage)) : $rel;
													$hrefurl = '/'.str_replace(array('&amp;', '='), array('-', '-'), $hrefurl).$this->config['urlext'];
													
													if($directories != '') {
														foreach($directories as $dir => $dirpages) {
															if(in_array($relpage, $dirpages)) {
																$hrefurl = '/'.$dir.$hrefurl;
																break;
															}
														}
													}
													$hrefurl = $relfolder.$hrefurl;
												}
												else {
													$hrefurl = '/index.php?wbpagefile='.$rel;
												}
												$hreflangs[$rellang] = $this->config['homepage'].$hrefurl;
											}
										}
									}
									else {
										$url = $modurl;
										$modified = '';
										$hreflangs = array();
									}
									
									$pages = explode('&', $url);
									$page = $pages[0];
									unset($pages);
									
									$checklang = $this->setlangasdirectory($page);
									$language = $checklang[0];
									if($checklang[1] == '') $checklang[1] = 'index';
									$mpage = $checklang[1];
									
									if($this->config['modrewrite'] == '1') {
										if($page != $mpage)
											$url = $mpage.substr($url, strlen($page));
										
										$url = '/'.str_replace(array('&amp;', '='), array('-', '-'), $url).$this->config['urlext'];
										
										if($directories != '') {
											foreach($directories as $dir => $dirpages) {
												if(in_array($page, $dirpages)) {
													$url = '/'.$dir.$url;
													break;
												}
											}
										}
										
										if($language != '')
											$url = '/'.$language.$url;
									}
									else {
										$url = '/index.php?wbpagefile='.$url;
									}
									
									$modulepages[$lang][] = array(
										'location' => $this->config['homepage'].$url,
										'modified' => $modified,
										'priority' => $priority_parents,
										'hreflangs' => $hreflangs
									);
								}
							}
						}
					}
					
					foreach($geturls as $lang => $modurls) {
						foreach($modurls as $modurl) {
							if(is_array($modurl) && array_key_exists('location', $modurl)) {
								$geturl = $modurl['location'];
								$modified = isset($modurl['modified']) ? $modurl['modified'] : '';
								$hreflangs = array();
								if(isset($modurl['hreflangs']) && count($modurl['hreflangs']) > 0) {
									foreach($modurl['hreflangs'] as $rellang => $rel)
									{
										if($this->config['modrewrite'] == '1') {
											$relurl = explode('&', $rel);
											$relpage = $relurl[0];
											unset($relurl);
											
											$checkrel = $this->setlangasdirectory($relpage);
											$relfolder = $checkrel[0] != '' ? '/'.$checkrel[0] : '';
											if($checkrel[1] == '') $checkrel[1] = 'index';
											$hrefurl = $relpage != $checkrel[1] ? $checkrel[1].substr($rel, strlen($relpage)) : $rel;
											
											$hrefurl = '/'.str_replace(array('&amp;', '='), array('-', '-'), $hrefurl).$this->config['urlext'];
											
											if($directories != '') {
												foreach($directories as $dir => $dirpages) {
													if(in_array($relpage, $dirpages)) {
														$hrefurl = '/'.$dir.$hrefurl;
														break;
													}
												}
											}
											$hrefurl = $relfolder.$hrefurl;
										}
										else {
											$hrefurl = '/index.php?wbpagefile='.$rel;
										}
										$hreflangs[$rellang] = $this->config['homepage'].$hrefurl;
									}
								}
							}
							else {
								$geturl = $modurl;
								$modified = '';
								$hreflangs = array();
							}
							
							$pages = explode('&', $geturl);
							$page = $pages[0];
							unset($pages);
								
							$checklang = $this->setlangasdirectory($page);
							$language = $checklang[0];
							if($checklang[1] == '') $checklang[1] = 'index';
							$mpage = $checklang[1];
							
							if($this->config['modrewrite'] == '1') {
								if($page != $mpage)
									$geturl = $mpage.substr($geturl, strlen($page));
								
								$geturl = '/'.str_replace(array('&amp;', '='), array('-', '-'), $geturl).$this->config['urlext'];
								
								if($directories != '') {
									foreach($directories as $dir => $dirpages) {
										if(in_array($page, $dirpages)) {
											$geturl = '/'.$dir.$geturl;
											break;
										}
									}
								}
								
								if($language != '')
									$geturl = '/'.$language.$geturl;
							}
							else {
								$geturl = '/index.php?wbpagefile='.$geturl;
							}
							
							$modulepages[$lang][] = array(
								'location' => $this->config['homepage'].$geturl,
								'modified' => $modified,
								'priority' => $priority_data,
								'hreflangs' => $hreflangs
							);
						}
					}
				}
				else {
					if($countparenturls > 0) {
						foreach($this->modparenturls as $parent => $modurl) {
							if($parent != 'root') {
								if(is_array($modurl) && array_key_exists('location', $modurl)) {
									$url = $modurl['location'];
									$modified = isset($modurl['modified']) ? $modurl['modified'] : '';
								}
								else {
									$url = $modurl;
									$modified = '';
								}
								
								$pages = explode('&', $url);
								$page = $pages[0];
								unset($pages);
								
								if($this->config['modrewrite'] == '1') {
									if($page == $this->config['startpage'])
										$url = 'index'.substr($url, strlen($page));
									$url = '/'.str_replace(array('&amp;', '='), array('-', '-'), $url).$this->config['urlext'];
									
									if($directories != '') {
										foreach($directories as $dir => $dirpages) {
											if(in_array($page, $dirpages)) {
												$url = '/'.$dir.$url;
												break;
											}
										}
									}
								}
								else {
									$url = '/index.php?wbpagefile='.$url;
								}
								
								$modulepages[] = array(
									'location' => $this->config['homepage'].$url,
									'modified' => $modified,
									'priority' => $priority_parents
								);
							}
						}
					}
					
					foreach($geturls as $modurl) {
						if(is_array($modurl) && array_key_exists('location', $modurl)) {
							$geturl = $modurl['location'];
							$modified = isset($modurl['modified']) ? $modurl['modified'] : '';
						}
						else {
							$geturl = $modurl;
							$modified = '';
						}
						
						$pages = explode('&', $geturl);
						$page = $pages[0];
						unset($pages);
						
						if($this->config['modrewrite'] == '1') {
							if($page == $this->config['startpage'])
								$geturl = 'index'.substr($geturl, strlen($page));
							$geturl = '/'.str_replace(array('&amp;', '='), array('-', '-'), $geturl).$this->config['urlext'];
							
							if($directories != '') {
								foreach($directories as $dir => $dirpages) {
									if(in_array($page, $dirpages)) {
										$geturl = '/'.$dir.$geturl;
										break;
									}
								}
							}
						}
						else {
							$geturl = '/index.php?wbpagefile='.$geturl;
						}
						
						$modulepages[] = array(
							'location' => $this->config['homepage'].$geturl,
							'modified' => $modified,
							'priority' => $priority_data
						);
					}
				}
			}
		}
        
        return $modulepages;
    }
    
    function getmodulessitemap($module)
	{
		$pages = explode(',', $module[1]);
		$modpages = array();
		if(array_key_exists('code', $this->langconf)) {
			foreach($pages as $page) {
				if(!in_array($page, $this->offlinepages) && !in_array($page, $this->blockedpages)) {
					foreach($this->langconf['code'] as $code) {
						if(in_array($page, $this->langconf['pages'][$code])) {
							/*
							if($this->config['modrewrite'] == '1' && (
							  ($this->config['langfolder'] == '1' && $page == $this->langconf['homes'][$code])
								|| 
							  ($this->config['langfolder'] != '1' && $page == $this->config['startpage'])
							)) {
								$modpages[$code] = 'index';//'';
							}
							else {
								$modpages[$code] = $page;
							}
							*/
							$modpages[$code] = $page;
						}
					}
				}
			}
		}
		else {
			if(!in_array($pages[0], $this->offlinepages) && !in_array($pages[0], $this->blockedpages)) {
				$modpages[] = $pages[0];
			}
		}
		
		$modulename = $module[0];
		$sitemapmodfile = $this->config['server_path'].'/modules/'.$modulename.'/sitemap.php';
		if(file_exists($sitemapmodfile)) {
			$classname = 'MMSitemapMod'.$modulename;
			$funcname = 'modulessitemap_'.$modulename;
			if(!class_exists($classname) && !function_exists($funcname))
				require $sitemapmodfile;
			
			if(function_exists($funcname)) {
				$result = $funcname($modpages);
			}
			if(class_exists('SQLite3') && class_exists($classname)) {
				$configname = $modulename.'_conf';
				if(!isset(${$configname}))
					require $this->config['server_path'].'/modules/'.$modulename.'/data/config.php';
				
				$modsitemapclass = new $classname;
				$modsitemapclass->modulepages = $modpages;
				$modsitemapclass->modname = $modulename;
				if(isset($module[2])) $modsitemapclass->basecat = $module[2];
				$modsitemapclass->langconf = $this->langconf;
				$modsitemapclass->serverpath = $this->config['server_path'];
				$modsitemapclass->fileconfig = ${$configname};
				$modsitemapclass->connectdb();
				
				if(array_key_exists('code', $this->langconf)) {
					if(array_key_exists('base', $modsitemapclass->fileconfig) && in_array('multilang', $modsitemapclass->fileconfig['base'])) {
						$result = $modsitemapclass->modsitemap();
						$this->modparenturls = $modsitemapclass->parenturls;
					}
					else {
						$langresult = array();
						$langparents = array();
						foreach($this->langconf['code'] as $code) {
							$modsitemapclass->pagelang = $code;
							$langresult = array_merge($langresult, $modsitemapclass->modsitemap());
							$langparents = array_merge($langparents, $modsitemapclass->parenturls);
						}
						$result = $langresult;
						$this->modparenturls = $langparents;
					}
				}
				else {
					$result = $modsitemapclass->modsitemap();
					$this->modparenturls = $modsitemapclass->parenturls;
				}
			}
		}
		
		return $result;
    }
	
    function get_root_modified($page)
	{
		$result = '';
		
		if(count($this->modparenturls) > 0) {
			foreach($this->modparenturls as $key => $modurl) {
				if($key == 'root') {
					foreach($modurl as $url => $modified) {
						if($url == $page) {
							$result = $modified;
							break;
							break;
						}
					}
				}
			}
		}
		
		return $result;
	}
    
    function contentpages()
	{
		$contentpages = array();
		
		$directories = '';
		if($this->config['directories'] == '1' && array_key_exists('pages', $this->directories) && count($this->directories['pages']) > 0) {
			$directories = $this->directories['pages'];
		}
		
		$pagesdir = $this->config['server_path'].'/content/pages';
		
		if(array_key_exists('code', $this->langconf)) {
			foreach($this->langconf['code'] as $code) {
				foreach($this->langconf['pages'][$code] as $page) {
					if(substr($page, 0, 5) != 'ajax_' && !in_array($page, $this->offlinepages) && !in_array($page, $this->blockedpages))
					{
						$relpages = array();
						if(array_key_exists('relations', $this->langconf)) {
							foreach($this->langconf['relations'] as $relation)
							{
								if(in_array($page, $relation)) {
									$relpages = $relation;
									break;
								}
							}
						}
						
						$hreflangs = array();
						if(count($relpages) > 0) {
							foreach($relpages as $lang => $rel)
							{
								if($this->config['modrewrite'] == '1') {
									$checkrel = $this->setlangasdirectory($rel);
									$hrefurl = $checkrel[1] == '' ? '/' : '/'.$checkrel[1].$this->config['urlext'];
									if($directories != '') {
										foreach($directories as $dir => $dirpages) {
											if(in_array($rel, $dirpages)) {
												$hrefurl = '/'.$dir.$hrefurl;
												break;
											}
										}
									}
									$relfolder = '';
									if($checkrel[0] != '' && !($lang == $this->config['defaultlang'] && $rel == $this->langconf['homes'][$lang])) {
										$relfolder = '/'.$checkrel[0];
									}
									$hrefurl = $relfolder.$hrefurl;
								}
								else {
									$hrefurl = '/index.php?wbpagefile='.$rel;
								}
								$hreflangs[$lang] = $this->config['homepage'].$hrefurl;
							}
						}
						
						$checklang = $this->setlangasdirectory($page);
						$language = $checklang[0];
						$cpage = $checklang[1];
						
						if($language == '' && $page == $this->config['startpage']) {
							$uri = $this->config['modrewrite'] == '1' ? '/' : '/index.php';
							$mod_modified = $this->get_root_modified($this->config['startpage']);
							$modified = $mod_modified == '' ? filemtime($pagesdir.'/'.$this->config['startpage']) : $mod_modified;
							$priority = '1.0';
						}
						else {
							$uri = $this->config['modrewrite'] == '1' ? ($cpage == '' ? '/' : '/'.$cpage.$this->config['urlext']) : '/index.php?wbpagefile='.$page;
							$mod_modified = $this->get_root_modified($page);
							$modified = $mod_modified == '' ? filemtime($pagesdir.'/'.$page) : $mod_modified;
							$priority = ($this->config['modrewrite'] == '1' && $cpage == '') ? '1.0' : '0.9'; //$cpage == 'index'
							if($this->config['modrewrite'] == '1') {
								if($directories != '') {
									foreach($directories as $dir => $dirpages) {
										if(in_array($page, $dirpages)) {
											$uri = '/'.$dir.$uri;
											break;
										}
									}
								}
								$langfolder = '';
								if($language != '' && !($code == $this->config['defaultlang'] && $page == $this->langconf['homes'][$code])) {
									$langfolder = '/'.$language;
								}
								$uri = $langfolder.$uri;
							}
						}
						
						$contentpages[$code][] = array(
							'location' => $this->config['homepage'].$uri,
							'modified' => $modified,
							'priority' => $priority,
							'hreflangs' => $hreflangs
						);
					}
				}
			}
		}
		else {
			$handle = opendir($pagesdir);
			while(false !== ($page = readdir($handle)))
			{
        		$ext = substr($page, -4);
				if($page != '.' && $page != '..' && $page != '.htaccess' && substr($page, 0, 5) != 'ajax_' && $ext != '.bak' && $ext != '.tmp' && $page != $this->config['errorpage'] && !in_array($page, $this->offlinepages) && !in_array($page, $this->blockedpages))
				{
					if($page == $this->config['startpage']) {
						//$uri = $this->config['modrewrite'] == '1' ? '/index'.$this->config['urlext'] : '/index.php';
						$uri = $this->config['modrewrite'] == '1' ? '/' : '/index.php';
						$mod_modified = $this->get_root_modified($this->config['startpage']);
						$modified = $mod_modified == '' ? filemtime($pagesdir.'/'.$this->config['startpage']) : $mod_modified;
						$priority = '1.0';
					}
					else {
						$uri = '/index.php?wbpagefile='.$page;
						$mod_modified = $this->get_root_modified($page);
						$modified = $mod_modified == '' ? filemtime($pagesdir.'/'.$page) : $mod_modified;
						$priority = '0.9';
						if($this->config['modrewrite'] == '1') {
							$uri = '/'.$page.$this->config['urlext'];
							if($directories != '') {
								foreach($directories as $dir => $dirpages) {
									if(in_array($page, $dirpages)) {
										$uri = '/'.$dir.$uri;
										break;
									}
								}
							}
						}
					}
					
					$contentpages[] = array(
						'location' => $this->config['homepage'].$uri,
						'modified' => $modified,
						'priority' => $priority
					);
				}
			}
		}
        
        return $contentpages;
    }
}


$mapfile = $webutler_config['server_path'].'/content/access/sitemapxml.txt';

if(!file_exists($mapfile) || filemtime($mapfile) < time()-$webutler_revisitafter) {
	$sitemap = new SitemapXML;
	$sitemap->config = $webutler_config;
	$sitemap->offlinepages = isset($webutler_offlinepages) ? $webutler_offlinepages : array();
	$sitemap->langconf = $webutler_config['languages'] == 1 && isset($webutler_langconf) ? $webutler_langconf : array();
	$sitemap->directories = $webutler_config['directories'] == 1 && isset($webutler_directories) ? $webutler_directories : array();
	$sitemap->modulesitemaps = isset($webutler_modulesitemaps) && count($webutler_modulesitemaps) > 0 ? $webutler_modulesitemaps : array();
	
	$map = '<?xml version="1.0" encoding="UTF-8"?'.'>'."\n";
	$map.= '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:xhtml="http://www.w3.org/1999/xhtml">'."\n";
	
	if(array_key_exists('code', $sitemap->langconf)) {
		foreach($sitemap->getpages() as $langs) {
			foreach($langs as $lang => $url) {
				$map.= '  <url>'."\n".
					'      <loc>'.$url['location'].'</loc>'."\n";
				if(isset($url['modified']) && $url['modified'] != '')
					$map.= '      <lastmod>'.gmdate("Y-m-d\TH:i:s\Z", $url['modified']).'</lastmod>'."\n";
				$map.= '      <changefreq>always</changefreq>'."\n".
					'      <priority>'.$url['priority'].'</priority>'."\n";
				if(isset($url['hreflangs']) && count($url['hreflangs']) > 0) {
					foreach($url['hreflangs'] as $hlang => $href) {
						$map.= '      <xhtml:link rel="alternate" hreflang="'.$hlang.'" href="'.$href.'" />'."\n";
					}
				}
				$map.= '  </url>'."\n";
			}
		}
	}
	else {
		foreach($sitemap->getpages() as $url) {
			$map.= '  <url>'."\n".
				'      <loc>'.$url['location'].'</loc>'."\n";
			if(isset($url['modified']) && $url['modified'] != '')
				$map.= '      <lastmod>'.gmdate("Y-m-d\TH:i:s\Z", $url['modified']).'</lastmod>'."\n";
			$map.= '      <changefreq>always</changefreq>'."\n".
				'      <priority>'.$url['priority'].'</priority>'."\n".
				'  </url>'."\n";
		}
	}

	$map.= '</urlset>'."\n";
	
	file_put_contents($mapfile, $map);
	
	$oldumask = umask(0);
	@chmod($mapfile, $webutler_config['chmod'][1]);
	umask($oldumask);
}

echo file_get_contents($mapfile);


